@@ -1,25 +1,11 @@
 /*
- * "$Id: gui.h 1397 2005-04-24 19:20:32Z mike $"
+ * GUI definitions for HTMLDOC, an HTML document processing program.
  *
- *   GUI definitions for HTMLDOC, an HTML document processing program.
+ * Copyright 2011-2018 by Michael R Sweet.
+ * Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   Copyright 1997-2005 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
+ * This program is free software.  Distribution and use rights are outlined in
+ * the file "COPYING".
  */
 
 /*
@@ -82,7 +68,8 @@ class GUI
 			*outputDirectory;
   Fl_Input		*outputPath;
   Fl_Button		*outputBrowse;
-  Fl_Round_Button	*typeHTML,
+  Fl_Round_Button	*typeEPUB,
+			*typeHTML,
 			*typeHTMLSep,
 			*typePS,
 			*typePDF;
@@ -189,7 +176,6 @@ class GUI
   Fl_Input		*path;
   Fl_Input		*proxy;
   Fl_Check_Button	*tooltips;
-  Fl_Check_Button	*modern_skin;
   Fl_Check_Button	*strict_html;
   Fl_Check_Button	*overflow_errors;
 
@@ -258,7 +244,6 @@ class GUI
 
   static void	htmlEditorCB(Fl_Widget *w, GUI *gui);
   static void	tooltipCB(Fl_Widget *w, GUI *gui);
-  static void	skinCB(Fl_Widget *w, GUI *gui);
   static void	saveOptionsCB(Fl_Widget *w, GUI *gui);
 
   static void	helpCB(Fl_Widget *w, GUI *gui);
@@ -296,8 +281,3 @@ class GUI
   void	show();
   int	visible() { return (window->visible()); }
 };
-
-
-/*
- * End of "$Id: gui.h 1397 2005-04-24 19:20:32Z mike $".
- */
