@@ -1,38 +1,11 @@
 /*
- * "$Id: html.cxx 1486 2006-05-31 15:27:28Z mike $"
+ * HTML exporting functions for HTMLDOC, a HTML document processing program.
  *
- *   HTML exporting functions for HTMLDOC, a HTML document processing program.
+ * Copyright 2011-2019 by Michael R Sweet.
+ * Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   Copyright 1997-2005 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
- *
- * Contents:
- *
- *   html_export()   - Export to HTML...
- *   count_headers() - Count the number of first and second level headings
- *                     in a document tree.
- *   write_header()  - Output the standard "header" for a HTML file.
- *   write_footer()  - Output the standard "footer" for a HTML file.
- *   write_all()     - Write all markup text for the given tree.
- *   get_title()     - Get the title string for the given document...
- *   add_link()      - Add a named link...
- *   find_link()     - Find a named link...
- *   compare_links() - Compare two named links.
+ * This program is free software.  Distribution and use rights are outlined in
+ * the file "COPYING".
  */
 
 /*
@@ -40,6 +13,7 @@
  */
 
 #include "htmldoc.h"
+#include "markdown.h"
 #include <ctype.h>
 
 
@@ -59,7 +33,7 @@ typedef struct
  */
 
 
-static int	num_links = 0,
+static size_t	num_links = 0,
 		alloc_links = 0;
 static link_t	*links;
 
@@ -122,15 +96,15 @@ html_export(tree_t *document,	/* I - Document to export */
 
   if (OutputFiles && TitleImage[0] && TitlePage &&
 #ifdef WIN32
-      stricmp(file_extension(TitleImage), "bmp") == 0 ||
-      stricmp(file_extension(TitleImage), "gif") == 0 ||
-      stricmp(file_extension(TitleImage), "jpg") == 0 ||
-      stricmp(file_extension(TitleImage), "png") == 0)
+      (stricmp(file_extension(TitleImage), "bmp") == 0 ||
+       stricmp(file_extension(TitleImage), "gif") == 0 ||
+       stricmp(file_extension(TitleImage), "jpg") == 0 ||
+       stricmp(file_extension(TitleImage), "png") == 0))
 #else
-      strcmp(file_extension(TitleImage), "bmp") == 0 ||
-      strcmp(file_extension(TitleImage), "gif") == 0 ||
-      strcmp(file_extension(TitleImage), "jpg") == 0 ||
-      strcmp(file_extension(TitleImage), "png") == 0)
+      (strcmp(file_extension(TitleImage), "bmp") == 0 ||
+       strcmp(file_extension(TitleImage), "gif") == 0 ||
+       strcmp(file_extension(TitleImage), "jpg") == 0 ||
+       strcmp(file_extension(TitleImage), "png") == 0))
 #endif // WIN32
     image_copy(TitleImage, file_find(TitleImage, Path), OutputPath);
 
@@ -142,6 +116,8 @@ html_export(tree_t *document,	/* I - Document to export */
   author    = htmlGetMeta(document, (uchar *)"author");
   copyright = htmlGetMeta(document, (uchar *)"copyright");
   docnumber = htmlGetMeta(document, (uchar *)"docnumber");
+  if (!docnumber)
+    docnumber = htmlGetMeta(document, (uchar *)"version");
 
  /*
   * Scan for all links in the document, and then update them...
@@ -290,8 +266,7 @@ write_header(FILE   **out,	/* IO - Output file */
 
   if (newfile)
   {
-    fputs("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" "
-          "\"http://www.w3.org/TR/REC-html40/loose.dtd\">\n", *out);
+    fputs("<!DOCTYPE html>\n", *out);
     fputs("<HTML>\n", *out);
     fputs("<HEAD>\n", *out);
     if (title != NULL)
@@ -302,7 +277,7 @@ write_header(FILE   **out,	/* IO - Output file */
       fprintf(*out, "<META NAME=\"copyright\" CONTENT=\"%s\">\n", copyright);
     if (docnumber != NULL)
       fprintf(*out, "<META NAME=\"docnumber\" CONTENT=\"%s\">\n", docnumber);
-    fprintf(*out, "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; CHARSET=iso-%s\">\n",
+    fprintf(*out, "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; CHARSET=%s\">\n",
             _htmlCharSet);
 
     if (OutputFiles)
@@ -327,19 +302,19 @@ write_header(FILE   **out,	/* IO - Output file */
     }
 
     fputs("<STYLE TYPE=\"text/css\"><!--\n", *out);
-    fprintf(*out, "BODY { font-family: %s }\n", families[_htmlBodyFont]);
-    fprintf(*out, "H1 { font-family: %s }\n", families[_htmlHeadingFont]);
-    fprintf(*out, "H2 { font-family: %s }\n", families[_htmlHeadingFont]);
-    fprintf(*out, "H3 { font-family: %s }\n", families[_htmlHeadingFont]);
-    fprintf(*out, "H4 { font-family: %s }\n", families[_htmlHeadingFont]);
-    fprintf(*out, "H5 { font-family: %s }\n", families[_htmlHeadingFont]);
-    fprintf(*out, "H6 { font-family: %s }\n", families[_htmlHeadingFont]);
-    fputs("SUB { font-size: smaller }\n", *out);
-    fputs("SUP { font-size: smaller }\n", *out);
-    fputs("PRE { font-family: monospace }\n", *out);
+    fprintf(*out, "BODY { font-family: %s; }\n", families[_htmlBodyFont]);
+    fprintf(*out, "H1 { font-family: %s; }\n", families[_htmlHeadingFont]);
+    fprintf(*out, "H2 { font-family: %s; }\n", families[_htmlHeadingFont]);
+    fprintf(*out, "H3 { font-family: %s; }\n", families[_htmlHeadingFont]);
+    fprintf(*out, "H4 { font-family: %s; }\n", families[_htmlHeadingFont]);
+    fprintf(*out, "H5 { font-family: %s; }\n", families[_htmlHeadingFont]);
+    fprintf(*out, "H6 { font-family: %s; }\n", families[_htmlHeadingFont]);
+    fputs("SUB { font-size: smaller; }\n", *out);
+    fputs("SUP { font-size: smaller; }\n", *out);
+    fputs("PRE { font-family: monospace; margin-left: 36pt; }\n", *out);
 
     if (!LinkStyle)
-      fputs("A { text-decoration: none }\n", *out);
+      fputs("A { text-decoration: none; }\n", *out);
 
     fputs("--></STYLE>\n", *out);
     fputs("</HEAD>\n", *out);
@@ -452,25 +427,28 @@ write_title(FILE  *out,		/* I - Output file */
             uchar *docnumber)	/* I - ID number for document */
 {
   FILE		*fp;		/* Title file */
-  const char	*title_file;	/* Location of title file */
+  const char	*title_ext,	/* Extension of title file */
+		*title_file;	/* Location of title file */
   tree_t	*t;		/* Title file document tree */
 
 
   if (out == NULL)
     return;
 
+  title_ext = file_extension(TitleImage);
+
 #ifdef WIN32
   if (TitleImage[0] &&
-      stricmp(file_extension(TitleImage), "bmp") != 0 &&
-      stricmp(file_extension(TitleImage), "gif") != 0 &&
-      stricmp(file_extension(TitleImage), "jpg") != 0 &&
-      stricmp(file_extension(TitleImage), "png") != 0)
+      stricmp(title_ext, "bmp") != 0 &&
+      stricmp(title_ext, "gif") != 0 &&
+      stricmp(title_ext, "jpg") != 0 &&
+      stricmp(title_ext, "png") != 0)
 #else
   if (TitleImage[0] &&
-      strcmp(file_extension(TitleImage), "bmp") != 0 &&
-      strcmp(file_extension(TitleImage), "gif") != 0 &&
-      strcmp(file_extension(TitleImage), "jpg") != 0 &&
-      strcmp(file_extension(TitleImage), "png") != 0)
+      strcmp(title_ext, "bmp") != 0 &&
+      strcmp(title_ext, "gif") != 0 &&
+      strcmp(title_ext, "jpg") != 0 &&
+      strcmp(title_ext, "png") != 0)
 #endif // WIN32
   {
     // Find the title page file...
@@ -490,7 +468,15 @@ write_title(FILE  *out,		/* I - Output file */
       return;
     }
 
-    t = htmlReadFile(NULL, fp, file_directory(TitleImage));
+#ifdef _WIN32
+    if (!stricmp(title_ext, "md"))
+#else
+    if (!strcmp(title_ext, "md"))
+#endif // _WIN32
+      t = mdReadFile(NULL, fp, file_directory(TitleImage));
+    else
+      t = htmlReadFile(NULL, fp, file_directory(TitleImage));
+
     htmlFixLinks(t, t, (uchar *)file_directory(TitleImage));
     fclose(fp);
 
@@ -606,7 +592,7 @@ write_node(FILE   *out,		/* I - Output file */
 	}
 	else
 	{
-	  if ((col + strlen((char *)t->data)) > 72 && col > 0)
+	  if ((col + (int)strlen((char *)t->data)) > 72 && col > 0)
 	  {
             putc('\n', out);
             col = 0;
@@ -1112,8 +1098,3 @@ update_links(tree_t *t,		/* I - Document tree */
     }
   }
 }
-
-
-/*
- * End of "$Id: html.cxx 1486 2006-05-31 15:27:28Z mike $".
- */
