@@ -1,40 +1,11 @@
 /*
- * "$Id: htmldoc.cxx 1480 2006-03-30 13:01:20Z mike $"
+ * Main entry for HTMLDOC, a HTML document processing program.
  *
- *   Main entry for HTMLDOC, a HTML document processing program.
+ * Copyright 2011-2019 by Michael R Sweet.
+ * Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   Copyright 1997-2006 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
- *
- * Contents:
- *
- *   main()            - Main entry for HTMLDOC.
- *   prefs_getrc()     - Get the rc file for preferences...
- *   prefs_load()      - Load HTMLDOC preferences...
- *   prefs_save()      - Save HTMLDOC preferences...
- *   prefs_set_paths() - Set HTMLDOC data/help paths...
- *   compare_strings() - Compare two command-line strings.
- *   load_book()       - Load a book file...
- *   parse_options()   - Parse options from a book file...
- *   read_file()       - Read a file into the current document.
- *   set_permissions() - Set the PDF permission bits...
- *   term_handler()    - Handle CTRL-C or kill signals...
- *   usage()           - Show program version and command-line options.
+ * This program is free software.  Distribution and use rights are outlined in
+ * the file "COPYING".
  */
 
 /*
@@ -43,6 +14,7 @@
 
 #define _HTMLDOC_CXX_
 #include "htmldoc.h"
+#include "markdown.h"
 #include "http.h"
 #include <ctype.h>
 #include <fcntl.h>
@@ -60,12 +32,16 @@
 #  include <sys/time.h>
 #endif // WIN32
 
+#ifdef __APPLE__
+#  include <libproc.h>
+#endif // __APPLE__
+
 #ifdef __EMX__
 extern "C" {
 const char *__XOS2RedirRoot(const char *);
 }
 #endif
- 
+
 
 /*
  * Local types...
@@ -83,6 +59,7 @@ static double	get_seconds(void);
 static int	load_book(const char *filename, tree_t **document,
 		          exportfunc_t *exportfunc, int set_nolocal = 0);
 static void	parse_options(const char *line, exportfunc_t *exportfunc);
+static const char *prefs_getrc(void);
 static int	read_file(const char *filename, tree_t **document,
 		          const char *path);
 static void	set_permissions(const char *p);
@@ -108,7 +85,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		*toc;			/* Table of contents */
   exportfunc_t	exportfunc;		/* Export function */
   const char	*extension;		/* Extension of output filename */
-  float		fontsize,		/* Base font size */
+  double	fontsize,		/* Base font size */
 		fontspacing;		/* Base font spacing */
   int		num_files;		/* Number of files provided on the command-line */
   double	start_time,		/* Start time */
@@ -171,7 +148,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     const char	*path_translated;	// PATH_TRANSLATED env var
     char	bookfile[1024];		// Book filename
 
-	
+
     // CGI mode implies the following options:
     //
     // --no-localfiles
@@ -191,7 +168,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     document      = NULL;
     exportfunc    = (exportfunc_t)pspdf_export;
     PSLevel       = 0;
-    PDFVersion    = 13;
+    PDFVersion    = 14;
     PDFPageMode   = PDF_DOCUMENT;
     PDFFirstPage  = PDF_PAGE_1;
 
@@ -486,30 +463,20 @@ main(int  argc,				/* I - Number of command-line arguments */
       i ++;
       if (i < argc)
       {
-        if (strcasecmp(argv[i], "ps1") == 0)
-        {
-	  exportfunc = (exportfunc_t)pspdf_export;
-	  PSLevel    = 1;
-	}
-        else if (strcasecmp(argv[i], "ps2") == 0 ||
-                 strcasecmp(argv[i], "ps") == 0)
-        {
-	  exportfunc = (exportfunc_t)pspdf_export;
-	  PSLevel    = 2;
-	}
-        else if (strcasecmp(argv[i], "ps3") == 0)
-        {
-	  exportfunc = (exportfunc_t)pspdf_export;
-	  PSLevel    = 3;
-	}
-        else if (strcasecmp(argv[i], "pdf14") == 0)
+        if (strcasecmp(argv[i], "epub") == 0)
+	  exportfunc = (exportfunc_t)epub_export;
+        else if (strcasecmp(argv[i], "html") == 0)
+          exportfunc = (exportfunc_t)html_export;
+        else if (strcasecmp(argv[i], "htmlsep") == 0)
+          exportfunc = (exportfunc_t)htmlsep_export;
+        else if (strcasecmp(argv[i], "pdf14") == 0 ||
+	         strcasecmp(argv[i], "pdf") == 0)
 	{
           exportfunc = (exportfunc_t)pspdf_export;
 	  PSLevel    = 0;
 	  PDFVersion = 14;
 	}
-        else if (strcasecmp(argv[i], "pdf13") == 0 ||
-	         strcasecmp(argv[i], "pdf") == 0)
+        else if (strcasecmp(argv[i], "pdf13") == 0)
 	{
           exportfunc = (exportfunc_t)pspdf_export;
 	  PSLevel    = 0;
@@ -528,10 +495,22 @@ main(int  argc,				/* I - Number of command-line arguments */
 	  PDFVersion  = 11;
 	  Compression = 0;
 	}
-        else if (strcasecmp(argv[i], "html") == 0)
-          exportfunc = (exportfunc_t)html_export;
-        else if (strcasecmp(argv[i], "htmlsep") == 0)
-          exportfunc = (exportfunc_t)htmlsep_export;
+        else if (strcasecmp(argv[i], "ps1") == 0)
+        {
+	  exportfunc = (exportfunc_t)pspdf_export;
+	  PSLevel    = 1;
+	}
+        else if (strcasecmp(argv[i], "ps2") == 0 ||
+                 strcasecmp(argv[i], "ps") == 0)
+        {
+	  exportfunc = (exportfunc_t)pspdf_export;
+	  PSLevel    = 2;
+	}
+        else if (strcasecmp(argv[i], "ps3") == 0)
+        {
+	  exportfunc = (exportfunc_t)pspdf_export;
+	  PSLevel    = 3;
+	}
 	else
 	  usage(argv[i - 1]);
       }
@@ -846,7 +825,7 @@ main(int  argc,				/* I - Number of command-line arguments */
       TocLevels = 0;
     else if (compare_strings(argv[i], "--no-truetype", 7) == 0)
     {
-      fputs("htmldoc: Warning, --no-truetype option superceded by --no-embedfonts!\n", stderr);
+      fputs("htmldoc: Warning, --no-truetype option superseded by --no-embedfonts!\n", stderr);
       EmbedFonts = 0;
     }
     else if (compare_strings(argv[i], "--no-xrxcomments", 6) == 0)
@@ -888,20 +867,22 @@ main(int  argc,				/* I - Number of command-line arguments */
 
         if ((extension = file_extension(argv[i])) != NULL)
         {
-          if (strcasecmp(extension, "ps") == 0)
+          if (strcasecmp(extension, "epub") == 0)
+            exportfunc = (exportfunc_t)epub_export;
+	  else if (strcasecmp(extension, "html") == 0)
+            exportfunc = (exportfunc_t)html_export;
+          else if (strcasecmp(extension, "pdf") == 0)
+	  {
+            exportfunc = (exportfunc_t)pspdf_export;
+	    PSLevel    = 0;
+          }
+          else if (strcasecmp(extension, "ps") == 0)
           {
 	    exportfunc = (exportfunc_t)pspdf_export;
 
 	    if (PSLevel == 0)
 	      PSLevel = 2;
 	  }
-          else if (strcasecmp(extension, "pdf") == 0)
-	  {
-            exportfunc = (exportfunc_t)pspdf_export;
-	    PSLevel    = 0;
-          }
-	  else if (strcasecmp(extension, "html") == 0)
-            exportfunc = (exportfunc_t)html_export;
         }
       }
       else
@@ -1103,7 +1084,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     }
     else if (compare_strings(argv[i], "--truetype", 4) == 0)
     {
-      fputs("htmldoc: Warning, --truetype option superceded by --embedfonts!\n", stderr);
+      fputs("htmldoc: Warning, --truetype option superseded by --embedfonts!\n", stderr);
 
       EmbedFonts = 1;
     }
@@ -1151,6 +1132,7 @@ main(int  argc,				/* I - Number of command-line arguments */
       _fsetmode(stdin, "b");
 #endif // WIN32 || __EMX__
 
+      _htmlCurrentFile = "(stdin)";
       htmlReadFile(file, stdin, ".");
 
       if (document == NULL)
@@ -1253,7 +1235,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     return (i);
   }
 #endif /* HAVE_LIBFLTK */
-    
+
  /*
   * We *must* have a document to process...
   */
@@ -1265,7 +1247,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Find the first one in the list...
   */
 
-  while (document->prev != NULL)
+  while (document && document->prev != NULL)
     document = document->prev;
 
   // Fix links...
@@ -1323,7 +1305,7 @@ main(int  argc,				/* I - Number of command-line arguments */
  * 'prefs_getrc()' - Get the rc file for preferences...
  */
 
-const char *
+static const char *
 prefs_getrc(void)
 {
 #ifdef WIN32
@@ -1511,8 +1493,6 @@ prefs_load(void)
         strlcpy(HTMLEditor, line + 7, sizeof(HTMLEditor));
       else if (strncasecmp(line, "TOOLTIPS=", 9) == 0)
         Tooltips = atoi(line + 9);
-      else if (strncasecmp(line, "MODERN=", 7) == 0)
-        ModernSkin = atoi(line + 7);
 #  endif // HAVE_LIBFLTK
     }
 
@@ -1621,7 +1601,6 @@ prefs_save(void)
 #ifdef HAVE_LIBFLTK
     fprintf(fp, "EDITOR=%s\n", HTMLEditor);
     fprintf(fp, "TOOLTIPS=%d\n", Tooltips);
-    fprintf(fp, "MODERN=%d\n", ModernSkin);
 #endif // HAVE_LIBFLTK
 
     fclose(fp);
@@ -1636,24 +1615,19 @@ prefs_save(void)
 void
 prefs_set_paths(void)
 {
+  //
+  // Get the installed directories...
+  //
+
 #ifdef WIN32			//// Do registry magic...
   HKEY		key;		// Registry key
   DWORD		size;		// Size of string
   static char	data[1024];	// Data directory
   static char	doc[1024];	// Documentation directory
   static char	path[4096];	// PATH environment variable
-#endif // WIN32
 
-
-  //
-  // Get the installed directories...
-  //
-
-#ifdef WIN32
   // Open the registry...
-  if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-                    "SOFTWARE\\Easy Software Products\\HTMLDOC", 0,
-                    KEY_READ, &key))
+  if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\HTMLDOC", 0, KEY_READ, &key))
   {
     // Grab the installed directories...
     size = sizeof(data);
@@ -1689,15 +1663,59 @@ prefs_set_paths(void)
         RegSetValueEx(key, "Path", 0, REG_EXPAND_SZ, (unsigned char *)path, strlen(path) + 1);
       }
   }
-#endif // WIN32
 
-#if defined(__EMX__) && defined(HAVE_LIBFLTK)
+#elif defined(__EMX__) && defined(HAVE_LIBFLTK)
   // If being installed within XFree86 OS/2 Environment
   // we can use those values which are overwritten by
   // the according environment variables.
   _htmlData = strdup(__XOS2RedirRoot("/XFree86/lib/X11/htmldoc"));
   GUI::help_dir = strdup(__XOS2RedirRoot("/XFree86/lib/X11/htmldoc/doc"));
-#endif // __EMX__ && HAVE_LIBFLTK
+
+#elif defined(__APPLE__) && defined(HAVE_LIBFLTK)
+  char          path[PROC_PIDPATHINFO_MAXSIZE + 1];
+                                	// Process path
+  int           pathlen;                // Length of path string
+  static char	resources[1024];	// Resources directory
+
+  if ((pathlen = proc_pidpath(getpid(), path, sizeof(path) - 1)) >= 0)
+  {
+    char        *ptr;           // Pointer into path
+
+    path[pathlen] = '\0';
+//    printf("proc_pidpath returned \"%s\"\n", path);
+
+    if ((ptr = strstr(path, "/Contents/MacOS")) != NULL)
+    {
+      *ptr = '\0';
+//      printf("Bundle path is \"%s\"\n", path);
+
+      snprintf(resources, sizeof(resources), "%s/Contents/Resources", path);
+      _htmlData     = resources;
+      GUI::help_dir = resources;
+    }
+  }
+//  else
+//    perror("proc_pidpath failed");
+
+#elif defined(__linux)
+  const char    *snap;                  // SNAP environment variable
+
+  // Support HTMLDOC as a snap package...
+  if ((snap = getenv("SNAP")) != NULL)
+  {
+    static char datadir[1024];          // Data directory
+
+    snprintf(datadir, sizeof(datadir), "%s/share/htmldoc", snap);
+    _htmlData = datadir;
+
+#  ifdef HAVE_LIBFLTK
+    static char docdir[1024];           // Documentation directory
+
+    snprintf(docdir, sizeof(docdir), "%s/share/doc/htmldoc", snap);
+    GUI::help_dir = docdir;
+#  endif // HAVE_LIBFLTK
+  }
+#endif // WIN32
 
   //
   // See if the installed directories have been overridden by
@@ -1718,16 +1736,16 @@ prefs_set_paths(void)
  * 'compare_strings()' - Compare two command-line strings.
  */
 
-static int			/* O - -1 or 1 = no match, 0 = match */
-compare_strings(const char *s,	/* I - Command-line string */
-                const char *t,	/* I - Option string */
-                int        tmin)/* I - Minimum number of unique chars in option */
+static int				/* O - -1 or 1 = no match, 0 = match */
+compare_strings(const char *s,		/* I - Command-line string */
+                const char *t,		/* I - Option string */
+                int        tmin)	/* I - Minimum number of unique chars in option */
 {
-  int	slen;			/* Length of command-line string */
+  size_t	slen;			/* Length of command-line string */
 
 
   slen = strlen(s);
-  if (slen < tmin)
+  if (slen < (size_t)tmin)
     return (-1);
   else
     return (strncmp(s, t, slen));
@@ -1862,7 +1880,7 @@ parse_options(const char   *line,	// I - Options from book file
   char		temp[1024],		// Option name
 		temp2[1024],		// Option value
 		*tempptr;		// Pointer into option
-  float		fontsize,		// Size of body text
+  double	fontsize,		// Size of body text
 		fontspacing;		// Spacing between lines
 
 
@@ -2055,10 +2073,37 @@ parse_options(const char   *line,	// I - Options from book file
 
     if (strcmp(temp, "-t") == 0 && !CGIMode)
     {
-      if (strcmp(temp2, "html") == 0)
+      if (strcmp(temp2, "epub") == 0)
+        *exportfunc = (exportfunc_t)epub_export;
+      else if (strcmp(temp2, "html") == 0)
         *exportfunc = (exportfunc_t)html_export;
       else if (strcmp(temp2, "htmlsep") == 0)
         *exportfunc = (exportfunc_t)htmlsep_export;
+      else if (strcmp(temp2, "pdf11") == 0)
+      {
+        *exportfunc = (exportfunc_t)pspdf_export;
+	PSLevel     = 0;
+	PDFVersion  = 11;
+      }
+      else if (strcmp(temp2, "pdf12") == 0)
+      {
+        *exportfunc = (exportfunc_t)pspdf_export;
+	PSLevel     = 0;
+	PDFVersion  = 12;
+      }
+      else if (strcmp(temp2, "pdf13") == 0)
+      {
+        *exportfunc = (exportfunc_t)pspdf_export;
+	PSLevel     = 0;
+	PDFVersion  = 13;
+      }
+      else if (strcmp(temp2, "pdf") == 0 ||
+               strcmp(temp2, "pdf14") == 0)
+      {
+        *exportfunc = (exportfunc_t)pspdf_export;
+	PSLevel     = 0;
+	PDFVersion  = 14;
+      }
       else if (strcmp(temp2, "ps1") == 0)
       {
         *exportfunc = (exportfunc_t)pspdf_export;
@@ -2075,31 +2120,6 @@ parse_options(const char   *line,	// I - Options from book file
         *exportfunc = (exportfunc_t)pspdf_export;
 	PSLevel     = 3;
       }
-      else if (strcmp(temp2, "pdf11") == 0)
-      {
-        *exportfunc = (exportfunc_t)pspdf_export;
-	PSLevel     = 0;
-	PDFVersion  = 11;
-      }
-      else if (strcmp(temp2, "pdf12") == 0)
-      {
-        *exportfunc = (exportfunc_t)pspdf_export;
-	PSLevel     = 0;
-	PDFVersion  = 12;
-      }
-      else if (strcmp(temp2, "pdf") == 0 ||
-               strcmp(temp2, "pdf13") == 0)
-      {
-        *exportfunc = (exportfunc_t)pspdf_export;
-	PSLevel     = 0;
-	PDFVersion  = 13;
-      }
-      else if (strcmp(temp2, "pdf14") == 0)
-      {
-        *exportfunc = (exportfunc_t)pspdf_export;
-	PSLevel     = 0;
-	PDFVersion  = 14;
-      }
     }
     else if (strcmp(temp, "--logo") == 0 ||
              strcmp(temp, "--logoimage") == 0)
@@ -2418,34 +2438,42 @@ read_file(const char *filename,		// I  - File/URL to read
 {
   FILE		*docfile;		// Document file
   tree_t	*file;			// HTML document file
-  const char	*realname;		// Real name of file
+  const char	*realname,		// Real name of file
+		*ext;			// Extension of filename
   char		base[1024];		// Base directory name of file
 
 
-  DEBUG_printf(("read_file(filename=\"%s\", document=%p, path=\"%s\")\n",
-                filename, document, path));
+  DEBUG_printf(("read_file(filename=\"%s\", document=%p, path=\"%s\")\n", filename, (void *)document, path));
 
   if ((realname = file_find(path, filename)) != NULL)
   {
     if ((docfile = fopen(realname, "rb")) != NULL)
     {
-     /*
-      * Read from a file...
-      */
-
+      // Prepare to read the file...
       if (Verbosity > 0)
         progress_error(HD_ERROR_NONE, "INFO: Reading %s...", filename);
 
       _htmlPPI = 72.0f * _htmlBrowserWidth / (PageWidth - PageLeft - PageRight);
 
       strlcpy(base, file_directory(filename), sizeof(base));
+      ext = file_extension(filename);
 
       file = htmlAddTree(NULL, MARKUP_FILE, NULL);
-      htmlSetVariable(file, (uchar *)"_HD_FILENAME",
-                      (uchar *)file_basename(filename));
+      htmlSetVariable(file, (uchar *)"_HD_URL", (uchar *)filename);
+      htmlSetVariable(file, (uchar *)"_HD_FILENAME", (uchar *)file_basename(filename));
       htmlSetVariable(file, (uchar *)"_HD_BASE", (uchar *)base);
 
-      htmlReadFile(file, docfile, base);
+      if (ext && !strcmp(ext, "md"))
+      {
+        // Read markdown from a file...
+        mdReadFile(file, docfile, base);
+      }
+      else
+      {
+        // Read HTML from a file...
+        _htmlCurrentFile = filename;
+        htmlReadFile(file, docfile, base);
+      }
 
       fclose(docfile);
 
@@ -2463,15 +2491,13 @@ read_file(const char *filename,		// I  - File/URL to read
     else
     {
       file = NULL;
-      progress_error(HD_ERROR_FILE_NOT_FOUND,
-                     "Unable to open \"%s\" for reading...", filename);
+      progress_error(HD_ERROR_FILE_NOT_FOUND, "Unable to open \"%s\" for reading...", filename);
     }
   }
   else
   {
     file = NULL;
-    progress_error(HD_ERROR_FILE_NOT_FOUND, "Unable to find \"%s\"...",
-                   filename);
+    progress_error(HD_ERROR_FILE_NOT_FOUND, "Unable to find \"%s\"...", filename);
   }
 
   return (file != NULL);
@@ -2532,6 +2558,8 @@ set_permissions(const char *p)		// I - Permission string
 
   if (Permissions != -4)
     Encryption = 1;
+
+  free(copyp);
 }
 
 
@@ -2562,8 +2590,17 @@ usage(const char *arg)			// I - Bad argument string
   if (CGIMode)
     puts("Content-Type: text/plain\r\n\r");
 
-  puts("HTMLDOC Version " SVERSION " Copyright 1997-2006 Easy Software Products, All Rights Reserved.");
-  puts("This software is based in part on the work of the Independent JPEG Group.");
+  puts("HTMLDOC Version " SVERSION " Copyright 2011-2019 by Michael R Sweet.");
+  puts("HTMLDOC is provided under the terms of the GNU General Public License and");
+  puts("comes with absolutely no warranty.  This software is based in part on the work");
+  puts("of the Independent JPEG Group.");
+  puts("");
+
+#ifdef HAVE_SSL
+  puts("This copy of HTMLDOC has been built to support both http: and https: URLs.");
+#else
+  puts("This copy of HTMLDOC has been built to support http: URLs only.");
+#endif /* HAVE_SSL */
   puts("");
 
   if (CGIMode)
@@ -2600,7 +2637,7 @@ usage(const char *arg)			// I - Bad argument string
     puts("  --book");
     puts("  --bottom margin{in,cm,mm}");
     puts("  --browserwidth pixels");
-    puts("  --charset {cp-874...1258,iso-8859-1...8859-15,koi8-r}");
+    puts("  --charset {cp-874...1258,iso-8859-1...-15,koi8-r,utf-8}");
     puts("  --color");
     puts("  --compression[=level]");
     puts("  --continuous");
@@ -2614,7 +2651,7 @@ usage(const char *arg)			// I - Bad argument string
     puts("  --fontsize {4.0..24.0}");
     puts("  --fontspacing {1.0..3.0}");
     puts("  --footer fff");
-    puts("  {--format, -t} {ps1,ps2,ps3,pdf11,pdf12,pdf13,pdf14,html,htmlsep}");
+    puts("  {--format, -t} {epub,html,htmlsep,pdf11,pdf12,pdf13,pdf14,ps1,ps2,ps3}");
     puts("  --gray");
     puts("  --header fff");
     puts("  --header1 fff");
@@ -2654,7 +2691,7 @@ usage(const char *arg)			// I - Bad argument string
     puts("  --numbered");
     puts("  --nup {1,2,4,6,9,16}");
     puts("  {--outdir, -d} dirname");
-    puts("  {--outfile, -f} filename.{ps,pdf,html}");
+    puts("  {--outfile, -f} filename.{epub,html,pdf,ps}");
     puts("  --overflow");
     puts("  --owner-password password");
     puts("  --pageduration {1.0..60.0}");
@@ -2704,12 +2741,8 @@ usage(const char *arg)			// I - Bad argument string
     puts("        l = logo image");
     puts("        t = title text");
     puts("        T = current time");
+    puts("        u = current file/URL");
   }
 
   exit(1);
 }
-
-
-/*
- * End of "$Id: htmldoc.cxx 1480 2006-03-30 13:01:20Z mike $".
- */
