@@ -1,37 +1,20 @@
-//
-// "$Id: htmlsep.cxx 1486 2006-05-31 15:27:28Z mike $"
-//
-//   Separated HTML export functions for HTMLDOC, a HTML document processing
-//   program.
-//
-//   Copyright 1997-2005 by Easy Software Products.
-//
-//   These coded instructions, statements, and computer programs are the
-//   property of Easy Software Products and are protected by Federal
-//   copyright law.  Distribution and use rights are outlined in the file
-//   "COPYING.txt" which should have been included with this file.  If this
-//   file is missing or damaged please contact Easy Software Products
-//   at:
-//
-//       Attn: ESP Licensing Information
-//       Easy Software Products
-//       44141 Airport View Drive, Suite 204
-//       Hollywood, Maryland 20636-3142 USA
-//
-//       Voice: (301) 373-9600
-//       EMail: info@easysw.com
-//         WWW: http://www.easysw.com
-//
-// Contents:
-//
-//   htmlsep_export() - Export to separated HTML files.
-//
+/*
+ * Separated HTML export functions for HTMLDOC, a HTML document processing
+ * program.
+ *
+ * Copyright 2011-2019 by Michael R Sweet.
+ * Copyright 1997-2010 by Easy Software Products.  All rights reserved.
+ *
+ * This program is free software.  Distribution and use rights are outlined in
+ * the file "COPYING".
+ */
 
-//
-// Include necessary headers.
-//
+/*
+ * Include necessary headers.
+ */
 
 #include "htmldoc.h"
+#include "markdown.h"
 #include <ctype.h>
 
 
@@ -51,12 +34,12 @@ typedef struct
 //
 
 // Heading strings used for filenames...
-static int	num_headings = 0,	// Number of headings
+static size_t	num_headings = 0,	// Number of headings
 		alloc_headings = 0;	// Allocated headings
 static uchar	**headings;		// Heading strings
 
 // Links in document - used to add the correct filename to the link
-static int	num_links = 0,		// Number of links
+static size_t	num_links = 0,		// Number of links
 		alloc_links = 0;	// Allocated links
 static link_t	*links;			// Links
 
@@ -100,13 +83,13 @@ int					// O - 0 = success, -1 = failure
 htmlsep_export(tree_t *document,	// I - Document to export
                tree_t *toc)		// I - Table of contents for document
 {
-  int	i;				// Looping var
-  int	heading;			// Current heading number
-  uchar	*title,				// Title text
-	*author,			// Author name
-	*copyright,			// Copyright text
-	*docnumber;			// Document number
-  FILE	*out;				// Output file
+  size_t	i;			// Looping var
+  int		heading;		// Current heading number
+  uchar		*title,			// Title text
+		*author,		// Author name
+		*copyright,		// Copyright text
+		*docnumber;		// Document number
+  FILE		*out;			// Output file
 
 
   // We only support writing to a directory...
@@ -126,15 +109,15 @@ htmlsep_export(tree_t *document,	// I - Document to export
 
   if (TitleImage[0] && TitlePage &&
 #ifdef WIN32
-      stricmp(file_extension(TitleImage), "bmp") == 0 ||
-      stricmp(file_extension(TitleImage), "gif") == 0 ||
-      stricmp(file_extension(TitleImage), "jpg") == 0 ||
-      stricmp(file_extension(TitleImage), "png") == 0)
+      (stricmp(file_extension(TitleImage), "bmp") == 0 ||
+       stricmp(file_extension(TitleImage), "gif") == 0 ||
+       stricmp(file_extension(TitleImage), "jpg") == 0 ||
+       stricmp(file_extension(TitleImage), "png") == 0))
 #else
-      strcmp(file_extension(TitleImage), "bmp") == 0 ||
-      strcmp(file_extension(TitleImage), "gif") == 0 ||
-      strcmp(file_extension(TitleImage), "jpg") == 0 ||
-      strcmp(file_extension(TitleImage), "png") == 0)
+      (strcmp(file_extension(TitleImage), "bmp") == 0 ||
+       strcmp(file_extension(TitleImage), "gif") == 0 ||
+       strcmp(file_extension(TitleImage), "jpg") == 0 ||
+       strcmp(file_extension(TitleImage), "png") == 0))
 #endif // WIN32
     image_copy(TitleImage, file_find(TitleImage, Path), OutputPath);
 
@@ -143,6 +126,8 @@ htmlsep_export(tree_t *document,	// I - Document to export
   author    = htmlGetMeta(document, (uchar *)"author");
   copyright = htmlGetMeta(document, (uchar *)"copyright");
   docnumber = htmlGetMeta(document, (uchar *)"docnumber");
+  if (!docnumber)
+    docnumber = htmlGetMeta(document, (uchar *)"version");
 
   // Scan for all links in the document, and then update them...
   num_links   = 0;
@@ -260,8 +245,7 @@ write_header(FILE   **out,	/* IO - Output file */
     return;
   }
 
-  fputs("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" "
-        "\"http://www.w3.org/TR/REC-html40/loose.dtd\">\n", *out);
+  fputs("<!DOCTYPE html>\n", *out);
   fputs("<HTML>\n", *out);
   fputs("<HEAD>\n", *out);
   if (title != NULL)
@@ -272,7 +256,7 @@ write_header(FILE   **out,	/* IO - Output file */
     fprintf(*out, "<META NAME=\"copyright\" CONTENT=\"%s\">\n", copyright);
   if (docnumber != NULL)
     fprintf(*out, "<META NAME=\"docnumber\" CONTENT=\"%s\">\n", docnumber);
-  fprintf(*out, "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; CHARSET=iso-%s\">\n",
+  fprintf(*out, "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; CHARSET=%s\">\n",
           _htmlCharSet);
 
   fputs("<LINK REL=\"Start\" HREF=\"index.html\">\n", *out);
@@ -287,24 +271,24 @@ write_header(FILE   **out,	/* IO - Output file */
     if (heading > 0)
       fprintf(*out, "<LINK REL=\"Prev\" HREF=\"%s.html\">\n", headings[heading - 1]);
 
-    if (heading < (num_headings - 1))
+    if ((size_t)heading < (num_headings - 1))
       fprintf(*out, "<LINK REL=\"Next\" HREF=\"%s.html\">\n", headings[heading + 1]);
   }
 
   fputs("<STYLE TYPE=\"text/css\"><!--\n", *out);
-  fprintf(*out, "BODY { font-family: %s }\n", families[_htmlBodyFont]);
-  fprintf(*out, "H1 { font-family: %s }\n", families[_htmlHeadingFont]);
-  fprintf(*out, "H2 { font-family: %s }\n", families[_htmlHeadingFont]);
-  fprintf(*out, "H3 { font-family: %s }\n", families[_htmlHeadingFont]);
-  fprintf(*out, "H4 { font-family: %s }\n", families[_htmlHeadingFont]);
-  fprintf(*out, "H5 { font-family: %s }\n", families[_htmlHeadingFont]);
-  fprintf(*out, "H6 { font-family: %s }\n", families[_htmlHeadingFont]);
-  fputs("SUB { font-size: smaller }\n", *out);
-  fputs("SUP { font-size: smaller }\n", *out);
-  fputs("PRE { font-family: monospace }\n", *out);
+  fprintf(*out, "BODY { font-family: %s; }\n", families[_htmlBodyFont]);
+  fprintf(*out, "H1 { font-family: %s; }\n", families[_htmlHeadingFont]);
+  fprintf(*out, "H2 { font-family: %s; }\n", families[_htmlHeadingFont]);
+  fprintf(*out, "H3 { font-family: %s; }\n", families[_htmlHeadingFont]);
+  fprintf(*out, "H4 { font-family: %s; }\n", families[_htmlHeadingFont]);
+  fprintf(*out, "H5 { font-family: %s; }\n", families[_htmlHeadingFont]);
+  fprintf(*out, "H6 { font-family: %s; }\n", families[_htmlHeadingFont]);
+  fputs("SUB { font-size: smaller; }\n", *out);
+  fputs("SUP { font-size: smaller; }\n", *out);
+  fputs("PRE { font-family: monospace; margin-left: 36pt; }\n", *out);
 
   if (!LinkStyle)
-    fputs("A { text-decoration: none }\n", *out);
+    fputs("A { text-decoration: none; }\n", *out);
 
   fputs("--></STYLE>\n", *out);
   fputs("</HEAD>\n", *out);
@@ -342,7 +326,7 @@ write_header(FILE   **out,	/* IO - Output file */
     if (heading > 0)
       fprintf(*out, "<A HREF=\"%s.html\">Previous</A>\n", headings[heading - 1]);
 
-    if (heading < (num_headings - 1))
+    if ((size_t)heading < (num_headings - 1))
       fprintf(*out, "<A HREF=\"%s.html\">Next</A>\n", headings[heading + 1]);
 
     fputs("<HR NOSHADE>\n", *out);
@@ -380,7 +364,7 @@ write_footer(FILE **out,	/* IO - Output file pointer */
     if (heading > 0)
       fprintf(*out, "<A HREF=\"%s.html\">Previous</A>\n", headings[heading - 1]);
 
-    if (heading < (num_headings - 1))
+    if ((size_t)heading < (num_headings - 1))
       fprintf(*out, "<A HREF=\"%s.html\">Next</A>\n", headings[heading + 1]);
   }
 
@@ -406,25 +390,28 @@ write_title(FILE  *out,		/* I - Output file */
             uchar *docnumber)	/* I - ID number for document */
 {
   FILE		*fp;		/* Title file */
-  const char	*title_file;	/* Location of title file */
+  const char	*title_ext,	/* Extension of title file */
+		*title_file;	/* Location of title file */
   tree_t	*t;		/* Title file document tree */
 
 
   if (out == NULL)
     return;
 
+  title_ext = file_extension(TitleImage);
+
 #ifdef WIN32
   if (TitleImage[0] &&
-      stricmp(file_extension(TitleImage), "bmp") != 0 &&
-      stricmp(file_extension(TitleImage), "gif") != 0 &&
-      stricmp(file_extension(TitleImage), "jpg") != 0 &&
-      stricmp(file_extension(TitleImage), "png") != 0)
+      stricmp(title_ext, "bmp") != 0 &&
+      stricmp(title_ext, "gif") != 0 &&
+      stricmp(title_ext, "jpg") != 0 &&
+      stricmp(title_ext, "png") != 0)
 #else
   if (TitleImage[0] &&
-      strcmp(file_extension(TitleImage), "bmp") != 0 &&
-      strcmp(file_extension(TitleImage), "gif") != 0 &&
-      strcmp(file_extension(TitleImage), "jpg") != 0 &&
-      strcmp(file_extension(TitleImage), "png") != 0)
+      strcmp(title_ext, "bmp") != 0 &&
+      strcmp(title_ext, "gif") != 0 &&
+      strcmp(title_ext, "jpg") != 0 &&
+      strcmp(title_ext, "png") != 0)
 #endif // WIN32
   {
     // Find the title page file...
@@ -444,7 +431,15 @@ write_title(FILE  *out,		/* I - Output file */
       return;
     }
 
-    t = htmlReadFile(NULL, fp, file_directory(TitleImage));
+#ifdef _WIN32
+    if (!stricmp(title_ext, "md"))
+#else
+    if (!strcmp(title_ext, "md"))
+#endif // _WIN32
+      t = mdReadFile(NULL, fp, file_directory(TitleImage));
+    else
+      t = htmlReadFile(NULL, fp, file_directory(TitleImage));
+
     htmlFixLinks(t, t, (uchar *)file_directory(TitleImage));
     fclose(fp);
 
@@ -536,7 +531,7 @@ write_doc(FILE   **out,			// I - Output file
     if (t->markup >= MARKUP_H1 && t->markup < (MARKUP_H1 + TocLevels) &&
         htmlGetVariable(t, (uchar *)"_HD_OMIT_TOC") == NULL)
     {
-      if (heading >= 0)
+      if (*heading >= 0)
         write_footer(out, *heading);
 
       (*heading) ++;
@@ -603,7 +598,7 @@ write_node(FILE   *out,		/* I - Output file */
 	}
 	else
 	{
-	  if ((col + strlen((char *)t->data)) > 72 && col > 0)
+	  if ((col + (int)strlen((char *)t->data)) > 72 && col > 0)
 	  {
             putc('\n', out);
             col = 0;
@@ -908,13 +903,13 @@ get_title(tree_t *doc)	/* I - Document tree */
 static void
 add_heading(tree_t *t)			// I - Heading node
 {
-  int	i,				// Looping var
-	count;				// Count of headings with this name
-  uchar	*heading,			// Heading text for this node
-	*ptr,				// Pointer into text
-	*ptr2,				// Second pointer into text
-	s[1024],			// New text if we have a conflict
-	**temp;				// New heading array pointer
+  size_t	i,			// Looping var
+		count;			// Count of headings with this name
+  uchar		*heading,		// Heading text for this node
+		*ptr,			// Pointer into text
+		*ptr2,			// Second pointer into text
+		s[1024],		// New text if we have a conflict
+		**temp;			// New heading array pointer
 
 
   // Start by getting the heading text...
@@ -941,7 +936,7 @@ add_heading(tree_t *t)			// I - Heading node
     {
       // Create a new instance of the heading...
       count ++;
-      snprintf((char *)s, sizeof(s), "%s%d", heading, count);
+      snprintf((char *)s, sizeof(s), "%s%d", heading, (int)count);
       ptr = s;
     }
 
@@ -1130,7 +1125,7 @@ update_links(tree_t *t,		/* I - Document tree */
       (*heading) ++;
 
     // Figure out the current filename based upon the current heading number...
-    if (!heading || *heading < 0 || *heading >= num_headings)
+    if (!heading || *heading < 0 || (size_t)*heading >= num_headings)
       filename = (uchar *)"noheading";
     else
       filename = headings[*heading];
@@ -1161,8 +1156,3 @@ update_links(tree_t *t,		/* I - Document tree */
     t = t->next;
   }
 }
-
-
-//
-// End of "$Id: htmlsep.cxx 1486 2006-05-31 15:27:28Z mike $".
-//
