@@ -1,32 +1,12 @@
 /*
- * "$Id: iso8859.cxx 1479 2006-03-30 03:25:07Z mike $"
+ * ISO-8859-1 conversion routines for HTMLDOC, an HTML document
+ * processing program.
  *
- *   ISO-8859-1 conversion routines for HTMLDOC, an HTML document
- *   processing program.
+ * Copyright 2011-2019 by Michael R Sweet.
+ * Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   Copyright 1997-2005 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
- *
- * Contents:
- *
- *   iso8859()     - Return the 8-bit character value of a glyph name.
- *   iso8859()     - Return the glyph name of an 8-bit character value.
- *   compare_lut() - Compare two glyphs.
+ * This program is free software.  Distribution and use rights are outlined in
+ * the file "COPYING".
  */
 
 /*
@@ -361,7 +341,7 @@ iso8859(uchar *name)			/* I - Glyph name */
 
     if ((glyph = _htmlGlyphsAll[ch]) == NULL)
     {
-      sprintf(uniglyph, "uni%04x", ch);
+      snprintf(uniglyph, sizeof(uniglyph), "uni%04x", ch);
       glyph = uniglyph;
     }
 
@@ -384,11 +364,20 @@ iso8859(uchar *name)			/* I - Glyph name */
 
           _htmlUnicode[newch] = ch;
 
-          // Reload font widths...
-	  htmlLoadFontWidths();
+          // Update font widths...
+          for (int typeface = 0; typeface < TYPE_MAX; typeface ++)
+	  {
+	    for (int style = 0; style < STYLE_MAX; style ++)
+	    {
+	      if (_htmlWidthsLoaded[typeface][style])
+	      {
+	        _htmlWidths[typeface][style][newch] = _htmlWidthsAll[typeface][style][ch];
+	      }
+	    }
+	  }
 
 	  // Return the new character...
-	  return (newch);
+	  return ((uchar)newch);
 	}
 
       // No room, return nul...
@@ -398,7 +387,7 @@ iso8859(uchar *name)			/* I - Glyph name */
       ch = newch;
   }
 
-  return (ch);
+  return ((uchar)ch);
 }
 
 
@@ -430,7 +419,7 @@ iso8859(uchar value)	/* I - ISO-8859-1 equivalent */
 
         if ((glyph = _htmlGlyphsAll[ch]) == NULL)
 	{
-	  sprintf(uniglyph, "uni%04x", ch);
+	  snprintf(uniglyph, sizeof(uniglyph), "uni%04x", ch);
 	  glyph = uniglyph;
 	}
 
@@ -451,12 +440,79 @@ iso8859(uchar value)	/* I - ISO-8859-1 equivalent */
     buf[1] = '\0';
   }
   else
-    sprintf((char *)buf, "&%s;", iso8859_names[value]->name);
+    snprintf((char *)buf, sizeof(buf), "&%s;", iso8859_names[value]->name);
 
   return (buf);
 }
 
 
+/*
+ * 'xhtml_entity()' - Return the UTF-8 character or XHTML entity.
+ */
+
+const uchar *                           /* O - XHTML string */
+xhtml_entity(uchar ch)                  /* I - Character */
+{
+  static uchar	buf[5];                 /* UTF-8 character buffer */
+
+
+  if (ch == '&')
+    return ((uchar *)"&amp;");
+  else if (ch == '<')
+    return ((uchar *)"&lt;");
+  else if (ch == '>')
+    return ((uchar *)"&gt;");
+  else if (ch == '\"')
+    return ((uchar *)"&quot;");
+  else if (ch >= ' ' && ch < 0x7f)
+  {
+    // US ASCII
+    buf[0] = ch;
+    buf[1] = '\0';
+
+    return (buf);
+  }
+  else
+  {
+    // Unicode -> UTF-8
+    int unich = _htmlUnicode[ch];
+
+    if (unich < 0x80)
+    {
+      // 1-byte US ASCII
+      buf[0] = (uchar)unich;
+      buf[1] = '\0';
+    }
+    else if (unich < 0x800)
+    {
+      // 2-byte UTF-8
+      buf[0] = (uchar)(0xc0 | (unich >> 6));
+      buf[1] = (uchar)(0x80 | (unich & 0x3f));
+      buf[2] = '\0';
+    }
+    else if (unich < 0x10000)
+    {
+      // 3-byte UTF-8
+      buf[0] = (uchar)(0xe0 | (unich >> 12));
+      buf[1] = (uchar)(0x80 | ((unich >> 6) & 0x3f));
+      buf[2] = (uchar)(0x80 | (unich & 0x3f));
+      buf[3] = '\0';
+    }
+    else
+    {
+      // 4-byte UTF-8
+      buf[0] = (uchar)(0xf0 | (unich >> 18));
+      buf[1] = (uchar)(0x80 | ((unich >> 12) & 0x3f));
+      buf[2] = (uchar)(0x80 | ((unich >> 6) & 0x3f));
+      buf[3] = (uchar)(0x80 | (unich & 0x3f));
+      buf[4] = '\0';
+    }
+
+    return (buf);
+  }
+}
+
+
 /*
  * 'compare_lut()' - Compare two glyphs.
  */
@@ -467,8 +523,3 @@ compare_lut(lut_t *a,	/* I - First glyph */
 {
   return (strcmp((char *)a->name, (char *)b->name));
 }
-
-
-/*
- * End of "$Id: iso8859.cxx 1479 2006-03-30 03:25:07Z mike $".
- */
