@@ -1,32 +1,12 @@
 /*
- * "$Id: toc.cxx 1397 2005-04-24 19:20:32Z mike $"
+ * Table of contents generator for HTMLDOC, a HTML document processing
+ * program.
  *
- *   Table of contents generator for HTMLDOC, a HTML document processing
- *   program.
+ * Copyright 2011-2019 by Michael R Sweet.
+ * Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   Copyright 1997-2005 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
- *
- * Contents:
- *
- *   toc_build()   - Build a table of contents of the given HTML tree.
- *   add_heading() - Add heading records to the given toc entry...
- *   parse_tree()  - Parse headings from the given tree...
+ * This program is free software.  Distribution and use rights are outlined in
+ * the file "COPYING".
  */
 
 /*
@@ -40,6 +20,7 @@
  * Local functions...
  */
 
+static void	add_heading(tree_t *toc, tree_t *heading);
 static void	parse_tree(tree_t *t);
 
 
@@ -107,13 +88,13 @@ toc_build(tree_t *tree)		/* I - Document tree */
  * 'add_heading()' - Add heading records to the given toc entry...
  */
 
-void
+static void
 add_heading(tree_t *toc,	/* I - Table of contents */
             tree_t *heading)	/* I - Heading entry */
 {
   while (heading != NULL)
   {
-    if (heading->child != NULL)
+    if (heading->markup != MARKUP_UNKNOWN && heading->child != NULL)
       add_heading(toc, heading->child);
     else if (heading->markup == MARKUP_NONE && heading->data != NULL)
       htmlAddTree(toc, MARKUP_NONE, heading->data);
@@ -223,45 +204,36 @@ parse_tree(tree_t *t)		/* I - Document tree */
 
           for (i = 0; i <= level; i ++)
           {
+            uchar	*baseptr = baselink + strlen((char *)baselink);
+            uchar	*headptr = heading + strlen((char *)heading);
+
             if (i == 0)
-              sprintf((char *)baselink + strlen((char *)baselink), "%d", TocDocCount);
+              snprintf((char *)baseptr, sizeof(baselink) - (size_t)(baseptr - baselink), "%d", TocDocCount);
             else
-              sprintf((char *)baselink + strlen((char *)baselink), "%d", heading_numbers[i]);
+              snprintf((char *)baseptr, sizeof(baselink) - (size_t)(baseptr - baselink), "%d", heading_numbers[i]);
 
             switch (heading_types[i])
             {
               case '1' :
-                  sprintf((char *)heading + strlen((char *)heading), "%d", heading_numbers[i]);
+                  snprintf((char *)headptr, sizeof(heading) - (size_t)(headptr - heading), "%d", heading_numbers[i]);
                   break;
               case 'a' :
                   if (heading_numbers[i] > 26)
-                    sprintf((char *)heading + strlen((char *)heading), "%c%c",
-                            'a' + (heading_numbers[i] / 26) - 1,
-                            'a' + (heading_numbers[i] % 26) - 1);
+                    snprintf((char *)headptr, sizeof(heading) - (size_t)(headptr - heading), "%c%c", 'a' + (heading_numbers[i] / 26) - 1, 'a' + (heading_numbers[i] % 26) - 1);
                   else
-                    sprintf((char *)heading + strlen((char *)heading), "%c",
-                            'a' + heading_numbers[i] - 1);
+                    snprintf((char *)headptr, sizeof(heading) - (size_t)(headptr - heading), "%c", 'a' + heading_numbers[i] - 1);
                   break;
               case 'A' :
                   if (heading_numbers[i] > 26)
-                    sprintf((char *)heading + strlen((char *)heading), "%c%c",
-                            'A' + (heading_numbers[i] / 26) - 1,
-                            'A' + (heading_numbers[i] % 26) - 1);
+                    snprintf((char *)headptr, sizeof(heading) - (size_t)(headptr - heading), "%c%c", 'A' + (heading_numbers[i] / 26) - 1, 'A' + (heading_numbers[i] % 26) - 1);
                   else
-                    sprintf((char *)heading + strlen((char *)heading), "%c",
-                            'A' + heading_numbers[i] - 1);
+                    snprintf((char *)headptr, sizeof(heading) - (size_t)(headptr - heading), "%c", 'A' + heading_numbers[i] - 1);
                   break;
               case 'i' :
-                  sprintf((char *)heading + strlen((char *)heading), "%s%s%s",
-                          hundreds[heading_numbers[i] / 100],
-                          tens[(heading_numbers[i] / 10) % 10],
-                          ones[heading_numbers[i] % 10]);
+                  snprintf((char *)headptr, sizeof(heading) - (size_t)(headptr - heading), "%s%s%s", hundreds[heading_numbers[i] / 100], tens[(heading_numbers[i] / 10) % 10], ones[heading_numbers[i] % 10]);
                   break;
               case 'I' :
-                  sprintf((char *)heading + strlen((char *)heading), "%s%s%s",
-                          HUNDREDS[heading_numbers[i] / 100],
-                          TENS[(heading_numbers[i] / 10) % 10],
-                          ONES[heading_numbers[i] % 10]);
+                  snprintf((char *)headptr, sizeof(heading) - (size_t)(headptr - heading), "%s%s%s", HUNDREDS[heading_numbers[i] / 100], TENS[(heading_numbers[i] / 10) % 10], ONES[heading_numbers[i] % 10]);
                   break;
             }
 
@@ -273,26 +245,37 @@ parse_tree(tree_t *t)		/* I - Document tree */
           }
 
          /*
-	  * See if we have an existing <A NAME=...> for this heading...
+	  * See if we have an existing <A NAME=...> or <A ID=...> for this
+	  * heading...
 	  */
 
           existing = NULL;
 
           if (t->parent != NULL && t->parent->markup == MARKUP_A)
+          {
 	    existing = htmlGetVariable(t->parent, (uchar *)"NAME");
 
+	    if (!existing)
+              existing = htmlGetVariable(t->parent, (uchar *)"ID");
+          }
+
 	  if (existing == NULL &&
               t->child != NULL && t->child->markup == MARKUP_A)
+          {
 	    existing = htmlGetVariable(t->child, (uchar *)"NAME");
 
+	    if (!existing)
+              existing = htmlGetVariable(t->child, (uchar *)"ID");
+          }
+
           if (existing != NULL &&
 	      strlen((char *)existing) >= 124)	/* Max size of link name */
 	    existing = NULL;
 
           if (existing != NULL)
-	    sprintf((char *)link, "#%s", existing);
+	    snprintf((char *)link, sizeof(link), "#%s", existing);
 	  else
-	    sprintf((char *)link, "#%s", baselink);
+	    snprintf((char *)link, sizeof(link), "#%s", baselink);
 
          /*
 	  * Number the headings as needed...
@@ -322,7 +305,7 @@ parse_tree(tree_t *t)		/* I - Document tree */
 		    htmlAddTree(heading_parents[last_level], MARKUP_UL, NULL);
 
               DEBUG_printf(("level=%d, last_level=%d, created new UL parent %p\n",
-	                    level, last_level, heading_parents[level]));
+	                    level, last_level, (void *)heading_parents[level]));
 	    }
 
             if (level == 0)
@@ -338,7 +321,7 @@ parse_tree(tree_t *t)		/* I - Document tree */
             else
               parent = htmlAddTree(heading_parents[level], MARKUP_LI, NULL);
 
-            DEBUG_printf(("parent=%p\n", parent));
+            DEBUG_printf(("parent=%p\n", (void *)parent));
 
             if ((var = htmlGetVariable(t, (uchar *)"_HD_OMIT_TOC")) != NULL)
 	      htmlSetVariable(parent, (uchar *)"_HD_OMIT_TOC", var);
@@ -396,8 +379,3 @@ parse_tree(tree_t *t)		/* I - Document tree */
     t = t->next;
   }
 }
-
-
-/*
- * End of "$Id: toc.cxx 1397 2005-04-24 19:20:32Z mike $".
- */
