@@ -1,36 +1,11 @@
 /*
- * "$Id: util.cxx 1397 2005-04-24 19:20:32Z mike $"
+ * Utility functions for HTMLDOC, a HTML document processing program.
  *
- *   Utility functions for HTMLDOC, a HTML document processing program.
+ * Copyright © 2011-2019 by Michael R Sweet.
+ * Copyright © 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   Copyright 1997-2005 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
- *
- * Contents:
- *
- *   format_number()   - Format a number into arabic numerals, roman numerals,
- *                       or letters.
- *   get_color()       - Get a standard color value...
- *   get_format()      - Convert an old "fff" format string to the new format.
- *   get_fmt()         - Convert a new format string to the old "fff" format.
- *   get_measurement() - Get a size measurement in inches, points, centimeters,
- *                       or millimeters.
- *   set_page_size()   - Set the output page size.
+ * This program is free software.  Distribution and use rights are outlined in
+ * the file "COPYING".
  */
 
 /*
@@ -93,38 +68,36 @@ format_number(int  n,		/* I - Number */
         if (n >= (26 * 26))
 	  buffer[0] = '\0';
         else if (n > 26)
-          sprintf(buffer, "%c%c", 'a' + (n / 26) - 1, 'a' + (n % 26) - 1);
+          snprintf(buffer, sizeof(buffer), "%c%c", 'a' + (n / 26) - 1, 'a' + (n % 26) - 1);
         else
-          sprintf(buffer, "%c", 'a' + n - 1);
+          snprintf(buffer, sizeof(buffer), "%c", 'a' + n - 1);
         break;
 
     case 'A' :
         if (n >= (26 * 26))
 	  buffer[0] = '\0';
         else if (n > 26)
-          sprintf(buffer, "%c%c", 'A' + (n / 26) - 1, 'A' + (n % 26) - 1);
+          snprintf(buffer, sizeof(buffer), "%c%c", 'A' + (n / 26) - 1, 'A' + (n % 26) - 1);
         else
-          sprintf(buffer, "%c", 'A' + n - 1);
+          snprintf(buffer, sizeof(buffer), "%c", 'A' + n - 1);
         break;
 
     case '1' :
-        sprintf(buffer, "%d", n);
+        snprintf(buffer, sizeof(buffer), "%d", n);
         break;
 
     case 'i' :
         if (n >= 1000)
 	  buffer[0] = '\0';
 	else
-          sprintf(buffer, "%s%s%s", hundreds[n / 100], tens[(n / 10) % 10],
-                  ones[n % 10]);
+          snprintf(buffer, sizeof(buffer), "%s%s%s", hundreds[n / 100], tens[(n / 10) % 10], ones[n % 10]);
         break;
 
     case 'I' :
         if (n >= 1000)
 	  buffer[0] = '\0';
 	else
-          sprintf(buffer, "%s%s%s", HUNDREDS[n / 100], TENS[(n / 10) % 10],
-                  ONES[n % 10]);
+          snprintf(buffer, sizeof(buffer), "%s%s%s", HUNDREDS[n / 100], TENS[(n / 10) % 10], ONES[n % 10]);
         break;
   }
 
@@ -180,7 +153,7 @@ get_color(const uchar *color,	/* I - Color attribute */
       if (!isxdigit(color[i]))
         break;
 
-    if (i == 6)
+    if (i == 3 || i == 6)
     {
       // Update the color name to be #RRGGBB instead of RRGGBB...
       tempcolor[0] = '#';
@@ -211,10 +184,29 @@ get_color(const uchar *color,	/* I - Color attribute */
     * RGB value in hex...
     */
 
-    i = strtol((char *)color + 1, NULL, 16);
-    rgb[0] = (i >> 16) / 255.0f;
-    rgb[1] = ((i >> 8) & 255) / 255.0f;
-    rgb[2] = (i & 255) / 255.0f;
+    i = (int)strlen((char *)color + 1);
+    if (i == 3)
+    {
+      i      = strtol((char *)color + 1, NULL, 16);
+      rgb[0] = (i >> 8) / 15.0f;
+      rgb[1] = ((i >> 4) & 15) / 15.0f;
+      rgb[2] = (i & 15) / 15.0f;
+    }
+    else if (i == 6)
+    {
+      i      = strtol((char *)color + 1, NULL, 16);
+      rgb[0] = (i >> 16) / 255.0f;
+      rgb[1] = ((i >> 8) & 255) / 255.0f;
+      rgb[2] = (i & 255) / 255.0f;
+    }
+    else if (defblack)
+    {
+      rgb[0] = rgb[1] = rgb[2] = 0.0f;
+    }
+    else
+    {
+      rgb[0] = rgb[1] = rgb[2] = 1.0f;
+    }
   }
   else
   {
@@ -318,6 +310,10 @@ get_format(const char *fmt,		// I - Old "fff" format
           formats[i] = strdup("$TIME");
           break;
 
+      case 'u' :
+          formats[i] = strdup("$URL");
+          break;
+
       default :
           formats[i] = NULL;
           break;
@@ -360,7 +356,7 @@ get_fmt(char **formats)			// I - New format strings
 
 
   // Safe because fmt is 4 chars long
-  strcpy(fmt, "...");
+  strlcpy(fmt, "...", sizeof(fmt));
 
   for (i = 0; i < 3; i ++)
     if (formats[i])
@@ -484,7 +480,7 @@ set_page_size(const char *size)	/* I - Page size string */
     PageWidth  = 595;
     PageLength = 792;
   }
-  else if (sscanf(size, "%fx%f%s", &width, &length, units) >= 2)
+  else if (sscanf(size, "%fx%f%254s", &width, &length, units) >= 2)
   {
    /*
     * Custom size...
@@ -512,8 +508,3 @@ set_page_size(const char *size)	/* I - Page size string */
     }
   }
 }
-
-
-/*
- * End of "$Id: util.cxx 1397 2005-04-24 19:20:32Z mike $".
- */
